/*
 * grvscored.c
 *
 * Simple highscore server -- run this from (x)inetd on ports
 * 22392 with option -s and 22393 with option -r
 */

#include <inttypes.h>
#include <limits.h>
#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/file.h>

#include "highscore.h"

int score_send(const char *file)
{
  FILE *f = fopen(file, "r");

  if ( !f )
    exit(1);

  flock(fileno(f), LOCK_SH);
  alarm(120);			/* Make sure we have a sane timeout */

  highscore_parse(f, fgets, 0);
  fclose(f);

  return highscore_write(stdout, fputs, 0) ? 1 : 0;
}

int score_recv(const char *file)
{
  FILE *f = fopen(file, "r+");
  FILE *t;
  char *tp;

  if ( !f || !(tp = alloca(strlen(file)+5)) )
    exit(1);

  sprintf(tp, "%s.tmp", file);
  if ( !(t = fopen(tp, "w")) )
    exit(1);

  flock(fileno(f), LOCK_EX);
  alarm(120);			/* Make sure we have a sane timeout */

  highscore_parse(f, fgets, 0);
  highscore_parse(stdin, fgets, 0);
  if ( highscore_write(t, fputs, 0) ) {
    unlink(tp);
    fclose(f);
    return 1;
  } else {
    rename(tp, file);
    fclose(f);
    return 0;
  }
}

int main(int argc, char *argv[])
{
  if ( argc != 3 ) {
    fprintf(stderr, "Usage: %s -s|r file\n", argv[0]);
    exit(1);
  }

  highscore_init();

  if ( argv[1][0] == '-' && argv[1][1] == 's' ) {
    return score_send(argv[2]);
  } else if ( argv[1][0] == '-' && argv[1][1] == 'r' ) {
    return score_recv(argv[2]);
  }

  exit(1);
}
